## PySESA (Python program for Spatially Explicit Spectral Analysis) 
## has been developed at the Grand Canyon Monitorinf & Research Center,
## U.S. Geological Survey
##
## Author: Daniel Buscombe
## Project homepage: <https://github.com/dbuscombe-usgs/pysesa>
##
##This software is in the public domain because it contains materials that originally came from 
##the United States Geological Survey, an agency of the United States Department of Interior. 
##For more information, see the official USGS copyright policy at 
##http://www.usgs.gov/visual-id/credit_usgs.html#copyright
##
## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
## See the GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program. If not, see <http://www.gnu.org/licenses/>.

##"""
## ___      ___ ___ ___   _     _   _ 
##| _ \_  _/ __| __/ __| /_\   (_) (_)
##|  _/ || \__ \ _|\__ \/ _ \   _   _ 
##|_|  \_, |___/___|___/_/ \_\ (_) (_)
##     |__/                           
##                _ __     
## _      _______(_) /____ 
##| | /| / / ___/ / __/ _ \
##| |/ |/ / /  / / /_/  __/
##|__/|__/_/  /_/\__/\___/ 
##                         

##+-+-+ +-+-+-+-+-+-+ +-+-+-+-+-+-+-+-+
##|b|y| |D|a|n|i|e|l| |B|u|s|c|o|m|b|e|
##+-+-+ +-+-+-+-+-+-+ +-+-+-+-+-+-+-+-+
##+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
##|d|b|u|s|c|o|m|b|e|@|u|s|g|s|.|g|o|v|
##+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
##+-+-+-+-+ +-+-+-+-+-+-+-+-+-+-+ +-+-+-+-+-+-+
##|U|.|S|.| |G|e|o|l|o|g|i|c|a|l| |S|u|r|v|e|y|
##+-+-+-+-+ +-+-+-+-+-+-+-+-+-+-+ +-+-+-+-+-+-+

##"""

# import libraries
import numpy as np
cimport numpy as np
cimport cython

# =========================================================
@cython.boundscheck(False)
@cython.cdivision(True)
@cython.wraparound(False)
@cython.nonecheck(False)
def txtwrite(str outfile, np.ndarray[np.float64_t, ndim=2] towrite, str header=None):
   '''
   Custom fast numpy array to comma-delimited ASCII txt file

   Syntax
   ----------
   () = pysesa.write.txtwrite(infile, towrite)

   Parameters
   ------------
   outfile : str
   	name of file to write to
   towrite : ndarray
   	ndarray containing Nx3 point cloud

   Other Parameters
   -----------------
   header : str, *optional* [default = None]
   	header string

   Returns
   ----------
   None

   '''
   with open(outfile, 'w') as f:

      np.savetxt(f, towrite[np.where(towrite[:,-1])[0],:], header = header, fmt=' '.join(['%8.6f,'] * np.shape(towrite)[1])[:-1]) 




## PySESA (Python program for Spatially Explicit Spectral Analysis) 
## has been developed at the Grand Canyon Monitorinf & Research Center,
## U.S. Geological Survey
##
## Author: Daniel Buscombe
## Project homepage: <https://github.com/dbuscombe-usgs/pysesa>
##
##This software is in the public domain because it contains materials that originally came from 
##the United States Geological Survey, an agency of the United States Department of Interior. 
##For more information, see the official USGS copyright policy at 
##http://www.usgs.gov/visual-id/credit_usgs.html#copyright
##
## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
## See the GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program. If not, see <http://www.gnu.org/licenses/>.

##"""
## ___      ___ ___ ___   _     _   _ 
##| _ \_  _/ __| __/ __| /_\   (_) (_)
##|  _/ || \__ \ _|\__ \/ _ \   _   _ 
##|_|  \_, |___/___|___/_/ \_\ (_) (_)
##     |__/                           
##                      __  _ __  _           
##    ____  ____ ______/ /_(_) /_(_)___  ____ 
##   / __ \/ __ `/ ___/ __/ / __/ / __ \/ __ \
##  / /_/ / /_/ / /  / /_/ / /_/ / /_/ / / / /
## / .___/\__,_/_/   \__/_/\__/_/\____/_/ /_/ 
##/_/                                         

##+-+-+ +-+-+-+-+-+-+ +-+-+-+-+-+-+-+-+
##|b|y| |D|a|n|i|e|l| |B|u|s|c|o|m|b|e|
##+-+-+ +-+-+-+-+-+-+ +-+-+-+-+-+-+-+-+
##+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
##|d|b|u|s|c|o|m|b|e|@|u|s|g|s|.|g|o|v|
##+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
##+-+-+-+-+ +-+-+-+-+-+-+-+-+-+-+ +-+-+-+-+-+-+
##|U|.|S|.| |G|e|o|l|o|g|i|c|a|l| |S|u|r|v|e|y|
##+-+-+-+-+ +-+-+-+-+-+-+-+-+-+-+ +-+-+-+-+-+-+

##"""

# import libraries
from __future__ import division
import numpy as np
cimport numpy as np
cimport cython
from scipy.spatial import cKDTree

# =========================================================
cdef class partition:

   '''
   Partition a Nx3 point cloud into M windows of nx3 points
   with specified spacing between centroids of adjacent windows
   and with specified overlap between windows.
   Implemented using a binary search tree for fast nearest neighbour 
   point check with boundary pruning

   Syntax
   ----------
   nr_pts = pysesa.partition(toproc, out, res, mxpts, minpts, prc_overlap).getdata()

   Parameters
   -----------
   toproc : ndarray
   	Nx3 point cloud

   Other Parameters
   -----------------
   out : float, *optional* [default = 0.5]
   	output grid resolution
   res : float, *optional* [default = 0.05]
   	spatial grid resolution to create a grid for the boundary pruning
   mxpts : float, *optional* [default = 1024]
   	maximum number of points allowed in a window
   minpts : float, *optional* [default = 16]
   	minimum number of points allowed in a window
   prc_overlap : float, *optional"  [default = 0]
        percentage overlap between windows

   Returns
   ----------
   self.data: list
   	list of M ndarrays, each containing n indices 
        of original point cloud, toproc, to partition space 
        to create M windows

   '''
   
   cdef object data, mytree, tree, tree2

   @cython.boundscheck(False)
   @cython.cdivision(True)
   @cython.wraparound(False)
   @cython.nonecheck(False)
   #==================================================
   def __init__(self, np.ndarray[np.float32_t, ndim=2] toproc, float out=0.5, float res=0.05, float mxpts=1024, float minpts=16, float prc_overlap=0):

      '''
      Partition a Nx3 point cloud into M windows of nx3 points
      with specified spacing between centroids of adjacent windows
      and with specified overlap between windows.
      Implemented using a binary search tree for fast nearest neighbour 
      point check with boundary pruning

      Syntax
      ----------
      nr_pts = pysesa.partition(toproc, out, res, mxpts, minpts, prc_overlap).getdata()

      Parameters
      ----------
      toproc : ndarray
   	   Nx3 point cloud

      Other Parameters
      ----------
      out : float, *optional* [default = 0.5]
   	   output grid resolution
      res : float, *optional* [default = 0.05]
   	   spatial grid resolution to create a grid for the boundary pruning
      mxpts : float, *optional* [default = 1024]
   	   maximum number of points allowed in a window
      minpts : float, *optional* [default = 16]
   	   minimum number of points allowed in a window
      prc_overlap : float, *optional"  [default = 0]
   	   percentage overlap between windows

      Returns
      ----------
      self.data: list
   	   list of M ndarrays, each containing n indices 
           of original point cloud, toproc, to partition space 
           to create M windows

      '''

      #get size of window
      cdef float win =  np.multiply(out/2, 1+(prc_overlap/100))

      #remove any rows containing NaNs
      toproc = toproc[~np.isnan(toproc).any(axis=1)]

      # pre-allocate arrays
      cdef list p
      cdef list allpoints
      cdef float xmin = np.min(toproc[:,0])
      cdef float xmax = np.max(toproc[:,0])
      cdef float ymin = np.min(toproc[:,1])
      cdef float ymax = np.max(toproc[:,1])
      cdef int orig_pts = len(toproc)
      cdef int lenx = np.ceil((xmax-xmin)/out)
      cdef int leny = np.ceil((ymax-ymin)/out)
      cdef int lenx2 = lenx/2    
      cdef int leny2 = leny/2   
      
      # no idea why this doesnt work
      #cdef np.ndarray[np.float64_t, ndim=1] x = np.empty(lenx, dtype=np.float64)
      #cdef np.ndarray[np.float64_t, ndim=1] y = np.empty(leny, dtype=np.float64)

      cdef np.ndarray[np.float64_t, ndim=1] xvec = np.empty((lenx2,), dtype=np.float64)
      cdef np.ndarray[np.float64_t, ndim=1] yvec = np.empty((leny2,), dtype=np.float64)   
   
      cdef np.ndarray[np.float64_t, ndim=2] xx = np.empty((int(np.ceil((ymax-ymin)/out)),int(np.ceil((xmax-xmin)/out))), dtype=np.float64)
      cdef np.ndarray[np.float64_t, ndim=2] yy = np.empty((int(np.ceil((ymax-ymin)/out)),int(np.ceil((xmax-xmin)/out))), dtype=np.float64)

      # create output grid
      x = np.arange(xmin, xmax, out)
      y = np.arange(ymin, ymax, out)
      xx, yy = np.meshgrid(x, y)
      p = list(np.vstack([xx.flatten(),yy.flatten()]).transpose())

      # format points for kd-tree
      allpoints = zip(toproc[:,0].ravel(), toproc[:,1].ravel())
        
      # find all points within 'out' metres of each centroid in p 
      xvec = np.arange(xmin-2*res,xmax+2*res)
      yvec = np.arange(ymin-2*res,ymax+2*res)            

      # pre-allocate more arrays
      cdef int k
      cdef tuple cx, cy
      cdef list indices_list
      cdef list w = []
      cdef list indices2 = []

      cdef np.ndarray[np.float64_t, ndim=2] dist = np.empty((len(p),mxpts), dtype=np.float64)
      cdef np.ndarray[np.int64_t, ndim=2] indices = np.empty((len(p),mxpts), dtype=np.int64)
      cdef np.ndarray[np.float64_t, ndim=2] xp = np.empty((len(yvec), len(xvec)), dtype=np.float64)
      cdef np.ndarray[np.float64_t, ndim=2] yp = np.empty((len(yvec), len(xvec)), dtype=np.float64)
      cdef np.ndarray[np.float64_t, ndim=1] dist2 = np.empty((len(xvec)*len(yvec),), dtype=np.float64)
      cdef np.ndarray[np.float64_t, ndim=1] dist3 = np.empty((len(p),), dtype=np.float64)
 
      # get the tree for the entire point cloud
      mytree = cKDTree(allpoints) 
   
      # largest inscribed square has side length = sqrt(2)*radius
      dist, indices = mytree.query(p,mxpts, distance_upper_bound=win)
      # remove any indices associated with 'inf' distance
      indices = np.squeeze(indices[np.where(np.all(np.isinf(dist),axis=1) ==  False),:])
      dist = np.squeeze(dist[np.where(np.all(np.isinf(dist),axis=1) ==  False),:])

      # define null indices
      indices[indices==len(allpoints)] = -999
      indices_list = indices.tolist()
      del indices

      # remove null records
      #for k in xrange(len(indices_list)):
      for k from 0 <= k < len(indices_list):
         indices_list[k] = [x for x in indices_list[k] if x!=-999]
      
      # get the centroids
      #for k in xrange(len(indices_list)):
      for k from 0 <= k < len(indices_list):
         w.append(np.mean(toproc[indices_list[k],:2],axis=0))

      cx,cy = zip(*w)
      del w

      yp, xp = np.meshgrid(yvec, xvec)

      # Use KDTree to answer the question: "which point of set (x,y) is the
      # nearest neighbors of those in (xp, yp)"
      tree = cKDTree(allpoints)
      dist2, _ = tree.query(np.c_[xp.ravel(), yp.ravel()], k=1)

      # Select points sufficiently far away (use hypoteneuse of the triangle made by res and res)
      tree2 = cKDTree(np.c_[xp.ravel()[(dist2 > np.hypot(res, res))], yp.ravel()[(dist2 > np.hypot(res, res))]])
      dist3, _ = tree.query(np.c_[cx,cy], distance_upper_bound=win) #distance_upper_bound=out)
      m2 = np.where(dist3 < out**2)[0]

      # do the pruning
      #for k in xrange(len(m2)):
      for k from 0 <= k < len(m2):
         if len(indices_list[m2[k]])>minpts:
            indices2.append(indices_list[m2[k]])

      self.data = indices2

   # =========================================================
   @cython.boundscheck(False)
   @cython.cdivision(True)
   @cython.wraparound(False)
   @cython.nonecheck(False)
   cpdef list getdata(self):
      '''
      Partition a Nx3 point cloud into M windows of nx3 points
      with specified spacing between centroids of adjacent windows
      and with specified overlap between windows.
      Implemented using a binary search tree for fast nearest neighbour 
      point check with boundary pruning

      Syntax
      ----------
      nr_pts = pysesa.partition.getdata()

      Parameters
      ----------
      self : instance
   	   pysesa.partition instance

      Returns
      ----------
      self.data: list
   	   list of M ndarrays, each containing n indices 
           of original point cloud, toproc, to partition space 
           to create M windows
      '''
      return self.data




## PySESA (Python program for Spatially Explicit Spectral Analysis) 
## has been developed at the Grand Canyon Monitorinf & Research Center,
## U.S. Geological Survey
##
## Author: Daniel Buscombe
## Project homepage: <https://github.com/dbuscombe-usgs/pysesa>
##
##This software is in the public domain because it contains materials that originally came from 
##the United States Geological Survey, an agency of the United States Department of Interior. 
##For more information, see the official USGS copyright policy at 
##http://www.usgs.gov/visual-id/credit_usgs.html#copyright
##
## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
## See the GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program. If not, see <http://www.gnu.org/licenses/>.

#"""                       
# ___      ___ ___ ___   _     _   _ 
#| _ \_  _/ __| __/ __| /_\   (_) (_)
#|  _/ || \__ \ _|\__ \/ _ \   _   _ 
#|_|  \_, |___/___|___/_/ \_\ (_) (_)
#     |__/                           
#                     __  _       __
#   _________  ____ _/ /_(_)___ _/ /
#  / ___/ __ \/ __ `/ __/ / __ `/ / 
# (__  ) /_/ / /_/ / /_/ / /_/ / /  
#/____/ .___/\__,_/\__/_/\__,_/_/   
#    /_/                            

#+-+-+ +-+-+-+-+-+-+ +-+-+-+-+-+-+-+-+
#|b|y| |D|a|n|i|e|l| |B|u|s|c|o|m|b|e|
#+-+-+ +-+-+-+-+-+-+ +-+-+-+-+-+-+-+-+
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#|d|b|u|s|c|o|m|b|e|@|u|s|g|s|.|g|o|v|
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#+-+-+-+-+ +-+-+-+-+-+-+-+-+-+-+ +-+-+-+-+-+-+
#|U|.|S|.| |G|e|o|l|o|g|i|c|a|l| |S|u|r|v|e|y|
#+-+-+-+-+ +-+-+-+-+-+-+-+-+-+-+ +-+-+-+-+-+-+
#"""

# import libraries
from __future__ import division
import numpy as np
cimport numpy as np
cimport cython
import RunningStats

# suppress divide and invalid warnings
import warnings
warnings.filterwarnings("ignore")
np.seterr(divide='ignore')
np.seterr(invalid='ignore')
np.seterr(over='ignore')
np.seterr(under='ignore')

# =========================================================
cdef class spatial:
   '''
   Calculate spatial statistics of a Nx3 point cloud

   Syntax
   ----------
   stats = pysesa.spatial(points).getdata()

   centroids = pysesa.spatial(points).getcentroid()

   stats = pysesa.spatial(points).getstats()

   Parameters
   -------------
   points : ndarray
   	Nx3 point cloud

   Returns [requested through .getdata()]
   ---------------------------------------
   self.data: list
   	x = centroid in horizontal coordinate

        y = centroid in laterial coordinate

        z_mean = centroid in amplitude

        z_max = max amplitude

        z_min = min amplitude

        z_range = range in amplitude

        sigma = standard deviation of amplitudes

        skewness = skewness of amplitudes

        kurtosis = skewness of amplitudes

        n = number of 3D coordinates


   Returns [requested through .getcentroid()]
   --------------------------------------------
   self.centroid: list
   	1x3 point cloud centroid [x,y,z]

   Returns [requested through .getstats()]
   ----------------------------------------
   self.stats: list
        z_mean = centroid in amplitude

        z_max = max amplitude

        z_min = min amplitude

        z_range = range in amplitude

        sigma = standard deviation of amplitudes

        skewness = skewness of amplitudes

        kurtosis = skewness of amplitudes

        n = number of 3D coordinates

   '''

   cdef object data, rs1, centroid, stats

   @cython.boundscheck(False)
   @cython.cdivision(True)
   @cython.wraparound(False)
   @cython.nonecheck(False)
   # =========================================================
   def __init__(self, np.ndarray[np.float64_t, ndim=2] points): 
      '''
      Calculate spatial statistics of a Nx3 point cloud

      Syntax
      ----------
      stats = pysesa.spatial(points).getdata()

      centroids = pysesa.spatial(points).getcentroid()

      stats = pysesa.spatial(points).getstats()

      Parameters
      -------------
      points : ndarray
   	   Nx3 point cloud

      Returns [requested through .getdata()]
      -----------------------------------------
      self.data: list
   	   x = centroid in horizontal coordinate

           y = centroid in laterial coordinate

           z_mean = centroid in amplitude

           z_max = max amplitude

           z_min = min amplitude

           z_range = range in amplitude

           sigma = standard deviation of amplitudes

           skewness = skewness of amplitudes

           kurtosis = skewness of amplitudes

           n = number of 3D coordinates


      Returns [requested through .getcentroid()]
      --------------------------------------------
      self.centroid: list
   	   1x3 point cloud centroid [x,y,z]

      Returns [requested through .getstats()]
      -----------------------------------------
      self.stats: list
           z_mean = centroid in amplitude

           z_max = max amplitude

           z_min = min amplitude

           z_range = range in amplitude

           sigma = standard deviation of amplitudes

           skewness = skewness of amplitudes

           kurtosis = skewness of amplitudes

           n = number of 3D coordinates

      '''
      
      cdef float x, y, z_mean,z_maz,z_min,z_range,sigma,skewness,kurtosis,n     
          
      x,y,z = self._get_centroid(points)
 
      z_mean,z_max,z_min,z_range,sigma,skewness,kurtosis,n = self._get_stats(points)

      self.stats = [z_mean,z_max,z_min,z_range,sigma,skewness,kurtosis,n]
      self.centroid = [x,y,z]

      #x,y,z_mean,z_max,z_min,z_range,sigma,skewness,kurtosis,n
      self.data = [x,y,z_mean,z_max,z_min,z_range,sigma,skewness,kurtosis,n] 

   # =========================================================
   @cython.boundscheck(False)
   @cython.cdivision(True)
   @cython.wraparound(False)
   @cython.nonecheck(False)
   cpdef list getdata(self):
      '''
      Calculate spatial statistics of a Nx3 point cloud

      Syntax
      ----------
      stats = pysesa.spatial.getdata()

      Parameters
      -------------
      self : instance
   	   pysesa.spatial instance

      Returns [requested through .getdata()]
      ---------------------------------------
      self.data: list
   	   x = centroid in horizontal coordinate

           y = centroid in laterial coordinate

           z_mean = centroid in amplitude

           z_max = max amplitude

           z_min = min amplitude

           z_range = range in amplitude

           sigma = standard deviation of amplitudes

           skewness = skewness of amplitudes

           kurtosis = skewness of amplitudes

           n = number of 3D coordinates

      '''
      return self.data

   # =========================================================
   @cython.boundscheck(False)
   @cython.cdivision(True)
   @cython.wraparound(False)
   @cython.nonecheck(False)
   cpdef list getcentroid(self):
      '''
      Calculate spatial statistics of a Nx3 point cloud

      Syntax
      ----------
      centroids = pysesa.spatial.getcentroid()

      Parameters
      -------------
      self : instance
   	   pysesa.spatial instance

      Returns [requested through .getcentroid()]
      --------------------------------------------
      self.centroid: list
   	   1x3 point cloud centroid [x,y,z]

      '''
      return self.centroid

   # =========================================================
   @cython.boundscheck(False)
   @cython.cdivision(True)
   @cython.wraparound(False)
   @cython.nonecheck(False)
   cpdef list getstats(self):
      '''
      Calculate spatial statistics of a Nx3 point cloud

      Syntax
      ----------
      stats = pysesa.spatial.getstats()

      Parameters
      ------------
      self : instance
   	   pysesa.spatial instance

      Returns [requested through .getstats()]
      -----------------------------------------
      self.stats: list
           z_mean = centroid in amplitude

           z_max = max amplitude

           z_min = min amplitude

           z_range = range in amplitude

           sigma = standard deviation of amplitudes

           skewness = skewness of amplitudes

           kurtosis = skewness of amplitudes

           n = number of 3D coordinates

      '''
      return self.stats

   # =========================================================
   @cython.boundscheck(False)
   @cython.cdivision(True)
   @cython.wraparound(False)
   @cython.nonecheck(False)
   cpdef list _get_centroid(self, np.ndarray[np.float64_t, ndim=2] points):
      '''
      Calculate centroid of a Nx3 point cloud

      Syntax
      ----------
      centroids = pysesa.spatial.getcentroid(points)

      Parameters
      ------------
      points : ndarray
   	   Nx3 point cloud

      Returns [requested through .getcentroid()]
      --------------------------------------------
      self.centroid: list
   	   1x3 point cloud centroid [x,y,z]

      '''
      cdef float x = np.min(points[:,0]) + ( np.max(points[:,0]) - np.min(points[:,0]) )/2      
      cdef float y = np.min(points[:,1]) + ( np.max(points[:,1]) - np.min(points[:,1]) )/2 
      cdef float z = np.min(points[:,2]) + ( np.max(points[:,2]) - np.min(points[:,2]) )/2 
                  
      return [x, y, z] 

   # =========================================================
   @cython.boundscheck(False)
   @cython.cdivision(True)
   @cython.wraparound(False)
   @cython.nonecheck(False)
   cpdef list _get_stats(self, np.ndarray[np.float64_t, ndim=2] points):
      '''
      Calculate spatial statistics of a Nx3 point cloud

      Syntax
      ----------
      stats = pysesa.spatial.getstats(points)

      Parameters
      ------------
      points : ndarray
   	   Nx3 point cloud

      Returns [requested through .getstats()]
      ----------------------------------------
      self.stats: list
           z_mean = centroid in amplitude

           z_max = max amplitude

           z_min = min amplitude

           z_range = range in amplitude

           sigma = standard deviation of amplitudes

           skewness = skewness of amplitudes

           kurtosis = skewness of amplitudes

           n = number of 3D coordinates

      '''
      rs1 = RunningStats.RunningStats()
      for k in points[:,2]:
         rs1.Push(k)
      #z_mean,z_max,z_min,z_range,sigma,skewness,kurtosis,n
      return [rs1.Mean(), np.max(points[:,2]), np.min(points[:,2]), np.max(points[:,2])-np.min(points[:,2]), rs1.StandardDeviation(), rs1.Skewness(), rs1.Kurtosis(), len(points)] 



